#include "StdAfx.h"
#include "PipeServer.h"
#include <process.h>

#define PIPE_TIMEOUT 500

CPipeServer::CPipeServer(void)
{
	m_pCmdProc = NULL;
	m_pParam = NULL;
	m_strEventName = L"";
	m_strPipeName = L"";

	m_hStopEvent = _CreateEvent(FALSE, FALSE, NULL);
	m_hThread = NULL;
}

CPipeServer::~CPipeServer(void)
{
	if( m_hThread != NULL ){
		::SetEvent(m_hStopEvent);
		// XbhI҂
		if ( ::WaitForSingleObject(m_hThread, 2000) == WAIT_TIMEOUT ){
			::TerminateThread(m_hThread, 0xffffffff);
		}
		CloseHandle(m_hThread);
		m_hThread = NULL;
	}
	::CloseHandle(m_hStopEvent);
	m_hStopEvent = NULL;
}

BOOL CPipeServer::StartServer(LPCWSTR lpwszEventName, LPCWSTR lpwszPipeName, CMD_CALLBACK_PROC pfnCmdProc, void* pParam)
{
	if( pfnCmdProc == NULL || pParam == NULL || lpwszEventName == NULL || lpwszPipeName == NULL ){
		return FALSE;
	}
	if( m_hThread != NULL ){
		return FALSE;
	}
	m_pCmdProc = pfnCmdProc;
	m_pParam = pParam;
	m_strEventName = lpwszEventName;
	m_strPipeName = lpwszPipeName;

	m_hThread = (HANDLE)_beginthreadex(NULL, 0, ServerThread, (LPVOID)this, CREATE_SUSPENDED, NULL);
	ResumeThread(m_hThread);

	return TRUE;
}

void CPipeServer::StopServer()
{
	if( m_hThread != NULL ){
		::SetEvent(m_hStopEvent);
		// XbhI҂
		if ( ::WaitForSingleObject(m_hThread, 2000) == WAIT_TIMEOUT ){
			::TerminateThread(m_hThread, 0xffffffff);
		}
		CloseHandle(m_hThread);
		m_hThread = NULL;
	}
}

UINT WINAPI CPipeServer::ServerThread(LPVOID pParam)
{
	CPipeServer* pSys = (CPipeServer*)pParam;

	HANDLE hPipe = NULL;
	HANDLE hEventCmdWait = NULL;
	HANDLE hEventConnect = NULL;
	HANDLE hEventArray[2];
	OVERLAPPED stOver;

	hEventCmdWait = _CreateEvent(FALSE, TRUE, CMD_CTRL_EVENT_WAIT);
	hEventConnect = _CreateEvent(FALSE, FALSE, pSys->m_strEventName.c_str());
	hEventArray[0] = pSys->m_hStopEvent;
	hEventArray[1] = _CreateEvent(FALSE, FALSE, NULL);
	
	while(1){
		if( hPipe == NULL ){
			while(1){
				hPipe = _CreateNamedPipe(pSys->m_strPipeName.c_str(), PIPE_ACCESS_DUPLEX | FILE_FLAG_WRITE_THROUGH | FILE_FLAG_OVERLAPPED,
				PIPE_TYPE_BYTE, 1,
				SEND_BUFF_SIZE, RES_BUFF_SIZE, PIPE_TIMEOUT);
				if( GetLastError() == ERROR_PIPE_BUSY ){
					Sleep(100);
				}else{
					break;
				}
			}

			ZeroMemory(&stOver, sizeof(OVERLAPPED));
			stOver.hEvent = hEventArray[1];
			ConnectNamedPipe(hPipe, &stOver);
			SetEvent(hEventConnect);
		}

		DWORD dwRes = WaitForMultipleObjects(2, hEventArray, FALSE, INFINITE);
		if( dwRes == WAIT_OBJECT_0 ){
			//STOP
			CloseHandle(hPipe);
			break;
		}else if( dwRes == WAIT_OBJECT_0+1 ){
			//ق̃T[o[ŏH
//			WaitForSingleObject(hEventCmdWait, INFINITE);
			//R}hM
			if( pSys->m_pCmdProc != NULL){
				CMD_STREAM stCmd;
				CMD_STREAM stRes;
				DWORD dwRead = 0;
				DWORD dwWrite = 0;
				do{
					if( ReadFile(hPipe, &stCmd, sizeof(DWORD)*2, &dwRead, NULL ) == FALSE ){
						break;
					}
					if( stCmd.dwSize > 0 ){
						stCmd.bData = new BYTE[stCmd.dwSize];
						DWORD dwReadNum = 0;
						while(dwReadNum < stCmd.dwSize ){
							DWORD dwReadSize = 0;
							if( stCmd.dwSize-dwReadNum < SEND_BUFF_SIZE ){
								dwReadSize = stCmd.dwSize-dwReadNum;
							}else{
								dwReadSize = SEND_BUFF_SIZE;
							}
							if( ReadFile(hPipe, stCmd.bData+dwReadNum, dwReadSize, &dwRead, NULL ) == FALSE ){
								break;
							}
							dwReadNum+=dwRead;
						}
						if( dwReadNum < stCmd.dwSize ){
							break;
						}
					}

					pSys->m_pCmdProc(pSys->m_pParam, &stCmd, &stRes);

					if( WriteFile(hPipe, &stRes, sizeof(DWORD)*2, &dwWrite, NULL ) == FALSE ){
						break;
					}
					if( stRes.dwSize > 0 ){
						if( stRes.bData == NULL ){
							break;
						}
						DWORD dwSendNum = 0;
						while(dwSendNum < stRes.dwSize ){
							DWORD dwSendSize = 0;
							if( stRes.dwSize-dwSendNum < RES_BUFF_SIZE ){
								dwSendSize = stRes.dwSize-dwSendNum;
							}else{
								dwSendSize = RES_BUFF_SIZE;
							}
							if( WriteFile(hPipe, stRes.bData+dwSendNum, dwSendSize, &dwWrite, NULL ) == FALSE ){
								break;
							}
							dwSendNum+=dwWrite;
						}
						if( dwSendNum < stRes.dwSize ){
							break;
						}
					}

				}while(stRes.dwParam == CMD_NEXT); //Emunp̌JԂ
			}
			FlushFileBuffers(hPipe);
			DisconnectNamedPipe(hPipe);
			CloseHandle(hPipe);
			hPipe = NULL;
//			SetEvent(hEventCmdWait);
		}
	}

	CloseHandle(hEventArray[1]);
	CloseHandle(hEventConnect);
	CloseHandle(hEventCmdWait);
	return 0;
}
